/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* parse configuration file routines
*
**************************************************************************/
#include "defs.h"
#include "lib.p"

PUBLIC boot_t boot;

/**************************************************************************
*
**************************************************************************/
PUBLIC int get_param(char *d_str, char *s_str, char *key, int max)
{
  char tok[STR_LEN + 1], param[STR_LEN + 1];

  sscanf(s_str, "%s %s", tok, param);
  if(!strcmp(tok, key))
  {
    char *s;

    s = strstr(s_str + strlen(tok), param);
    if(s != NULL)
      strncpy2(d_str, s, max);
    s = strchr(d_str, '\n');
    if(s != NULL) *s = 0;
    s = strchr(d_str, '\r');
    if(s != NULL) *s = 0;
    return 1;
  }

  return 0;
}

/**************************************************************************
*
**************************************************************************/
PRIVATE int parse_int(char c)
{
  switch(c)
  {
    case 'h':
      return INTERVAL_H;

    case 'd':
      return INTERVAL_D;

    case 'm':
      return INTERVAL_M;
  }

  return -1;
}

PRIVATE int parse_dir(char c)
{
  switch(c)
  {
    case 'I':
      return DIR_I;

    case 'O':
      return DIR_O;

    case 'u':
      return DIR_u;

    case 'i':
      return DIR_i;

    case 'o':
      return DIR_o;
  }

  return -1;
}

PRIVATE int parse_prot(char c)
{
  switch(c)
  {
    case 't':
      return PROT_TCP;

    case 'u':
      return PROT_UDP;

    case 'o':
      return PROT_OTH;

    case '*':
      return PROT_ALL;
  }

  return -1;
}

PRIVATE int parse_cond(char c)
{
  switch(c)
  {
    case '>':
      return COND_MORE;

    case '<':
      return COND_LESS;
  }

  return -1;
}

void parse_addr(addr_t *e, char *addr_str, char *mask_str)
{
  struct in_addr in;

  inet_aton(addr_str, &in);
  e->addr = ntohl(in.s_addr);
  inet_aton(mask_str, &in);
  e->mask = ntohl(in.s_addr);
  e->addr &= e->mask;
}

/**************************************************************************
* load conf file (sub funcs)
**************************************************************************/
PRIVATE int eserv_append_cmp_func(void *n1, void *n2)
{
  return cmp_16(((eserv_t *)n1)->port, ((eserv_t *)n2)->port);
}

PRIVATE int zone_append_cmp_func(void *n1, void *n2)
{
  return ((zone_t *)n1)->no - ((zone_t *)n2)->no;
}

PRIVATE int debug_append_cmp_func(void *n1, void *n2)
{
  return cmp_64(((addr_key_t *)n1)->key, ((addr_key_t *)n2)->key);
}

/**************************************************************************
* load conf file
**************************************************************************/
PUBLIC void read_boot_conf(void)
{
  FILE           *fp;
  char           str[STR_LEN + 1], substr[STR_LEN + 1],
                 addr_str[STR_LEN + 1], addr_str2[STR_LEN + 1];

  void parse_err(void)
  {
    char str2[STR_LEN + 1];

    strip_cr(str);
    sprintf(str2, " error parsing conf file- '%s'", str);
    log_m('E', NULL, str2);
    exit(-1);
  }

  void add_dosa(ll_t **ll, int *c)
  {
    dosa_t e;

    if(sscanf(substr, "%d %d", &(e.limit), &(e.zone)) != 2)
      parse_err();
    e.no = (*c)++;
    strncpy2(e.str, substr, STR_LEN);
    ll_append_unsorted(ll, &e, sizeof(dosa_t));
  }

  log_m('I', NULL, " loading conf file");
  if((fp = fopen(IPTRK_CONF, "r")) == NULL)
  {
    log_e(__func__, "::fopen()");
    exit(-1);
  }

  memset(&boot, 0, sizeof(boot_t));
  for(;;)
  {
    if(feof(fp))
      break;

    if(fgets(str, STR_LEN, fp) == NULL)
      break;

    if((str[0] == '#') || (str[0] < 32))
      continue;

    if(get_param(substr, str, "wait_delay:", STR_LEN))
    {
      if(sscanf(substr, "%d", &(boot.wait_delay)) != 1)
        parse_err();
      continue;
    }

    if(get_param(substr, str, "promisc:", STR_LEN))
    {
      static int c;

      if(substr[0] == 0)
        parse_err();

      if(c < BOOT_T_NIFS)
      {
        strncpy2(boot.nifs[c], substr, STR_LEN);
        c++;
      }
      continue;
    }

    if(get_param(substr, str, "watch_time:", STR_LEN))
    {
      int x;

      if(sscanf(substr, "%d", &x) != 1)
        parse_err();
      boot.watch_time = (60 * 60) / x;
      continue;
    }

    if(get_param(substr, str, "email:", STR_LEN))
    {
      static int c;

      if(substr[0] == 0)
        parse_err();

      if(c < BOOT_T_EMAIL_LIM)
      {
        strncpy2(boot.email[c], substr, STR_LEN);
        c++;
      }
      continue;
    }

    if(get_param(substr, str, "purge:", STR_LEN))
    {
      char c;

      if(sscanf(substr, "%c", &c) != 1)
        parse_err();
      boot.purge = parse_int(c);
      if(boot.purge == -1)
        parse_err();
      continue;
    }

    if(get_param(substr, str, "archive:", STR_LEN))
    {
      char c;

      if(sscanf(substr, "%c", &c) != 1)
        parse_err();
      boot.archive = parse_int(c);
      if(boot.archive == -1)
        parse_err();
      continue;
    }

    if(get_param(substr, str, "cancel_gram:", STR_LEN))
    {
      char c;

      if(sscanf(substr, "%c", &c) != 1)
        parse_err();
      if(c == 'y')
        boot.cancel_gram = 1;
      continue;
    }

    if(get_param(substr, str, "db_limit:", STR_LEN))
    {
      if(sscanf(substr, "%ld", &(boot.db_limit)) != 1)
        parse_err();
      continue;
    }

    if(get_param(substr, str, "our_net:", STR_LEN))
    {
      our_net_t e;

      if(sscanf(substr, "%s %s %s", str, addr_str, addr_str2) != 3)
        parse_err();
      strncpy2(e.dev, str, STR_SMALL);
      parse_addr(&(e.addr), addr_str, addr_str2);
      ll_append_unsorted(&(boot.our_net_ll), &e, sizeof(our_net_t));
      continue;
    }

    if(get_param(substr, str, "service:", STR_LEN))
    {
      eserv_t e;

      if(sscanf(substr, "%hu", &(e.port)) != 1)
        parse_err();

      treap_append(
        &(boot.eserv_bt), &e, sizeof(eserv_t), eserv_append_cmp_func);
      continue;
    }

    if(get_param(substr, str, "zone:", STR_LEN))
    {
      zone_t     e;
      char       day_s_str[STR_SMALL + 1],
                 day_e_str[STR_SMALL + 1];
      static int c;

      if(sscanf(substr, "%s %s %d %d",
        day_s_str, day_e_str, &(e.hr_s), &(e.hr_e)) != 4)
      {
        parse_err();
      }

      e.no = c++;
      e.day_s = day2i(day_s_str);
      e.day_e = day2i(day_e_str);
      if(e.day_s == -1 || e.day_e == -1)
        parse_err();

      treap_append(
        &(boot.zone_bt), &e, sizeof(zone_t), zone_append_cmp_func);
      continue;
    }

    if(get_param(substr, str, "dosa_addr:", STR_LEN))
    {
      static int c;

      add_dosa(&(boot.dosa_addr_ll), &c);
      continue;
    }

    if(get_param(substr, str, "dosa_port:", STR_LEN))
    {
      static int c;

      add_dosa(&(boot.dosa_port_ll), &c);
      continue;
    }

    if(get_param(substr, str, "watch:", STR_LEN))
    {
      watch_t    e;
      char       dir, prot, cond;
      static int c;

      if(sscanf(substr, "%s %s %c %c %d %c %llu %d",
        addr_str, addr_str2, &dir, &prot,
        &(e.service), &cond, &(e.limit), &(e.zone)) != 8)
      {
        parse_err();
      }

      e.no = c++;

      parse_addr(&(e.addr), addr_str, addr_str2);

      e.dir = parse_dir(dir);
      if(e.dir == -1)
        parse_err();

      e.prot = parse_prot(prot);
      if(e.prot == -1)
        parse_err();

      e.cond = parse_cond(cond);
      if(e.cond == -1)
        parse_err();

      strncpy2(e.str, substr, STR_LEN);

      ll_append_unsorted(&(boot.watch_ll), &e, sizeof(watch_t));
      continue;
    }

    if(get_param(substr, str, "watch_default:", STR_LEN))
    {
      watch_t    e;
      char       dir, prot, cond;
      static int c;

      if(sscanf(substr, "%c %c %d %c %llu %d",
        &dir, &prot, &(e.service), &cond, &(e.limit), &(e.zone)) != 6)
      {
        parse_err();
      }

      e.no = c++;

      e.dir = parse_dir(dir);
      if(e.dir == -1)
        parse_err();

      e.prot = parse_prot(prot);
      if(e.prot == -1)
        parse_err();

      e.cond = parse_cond(cond);
      if(e.cond == -1)
        parse_err();

      strncpy2(e.str, substr, STR_LEN);

      ll_append_unsorted(&(boot.watch_default_ll), &e, sizeof(watch_t));
      continue;
    }

    if(get_param(substr, str, "watch_private:", STR_LEN))
    {
      watch_private_t e;
      char            prot;
      static int      c;

      if(sscanf(substr, "%s %s %c %d %d",
        addr_str, addr_str2, &prot, &(e.service), &(e.zone)) != 5)
      {
        parse_err();
      }

      e.no = c++;

      parse_addr(&(e.addr), addr_str, addr_str2);

      e.prot = parse_prot(prot);
      if(e.prot == -1)
        parse_err();

      strncpy2(e.str, substr, STR_LEN);

      ll_append_unsorted(&(boot.watch_private_ll), &e, sizeof(watch_private_t));
      continue;
    }

    if(get_param(substr, str, "e_report:", STR_LEN))
    {
      addr_t e;

      if(sscanf(substr, "%s %s", addr_str, addr_str2) != 2)
        parse_err();
      parse_addr(&e, addr_str, addr_str2);
      ll_append_unsorted(&(boot.extra_report_ll), &e, sizeof(addr_t));
      continue;
    }

    if(get_param(substr, str, "debug:", STR_LEN))
    {
      addr_key_t     e;
      struct in_addr in;

      if(sscanf(substr, "%s %s", addr_str, addr_str2) != 2)
        parse_err();
      inet_aton(addr_str, &in);
      e.addr.s = ntohl(in.s_addr);
      inet_aton(addr_str2, &in);
      e.addr.d = ntohl(in.s_addr);
      treap_append(
        &(boot.debug_bt), &e, sizeof(addr_key_t), debug_append_cmp_func);
      continue;
    }
  }

  fclose(fp);
}
